@@ -247,6 +247,16 @@ void  func505_jac(const real_1d_array &x, real_1d_array &fi, real_2d_array &jac,
     jac[1][0] = 2*(x[0]-x0);
 }
 
+void file_put_contents(const char *filename, const char *contents)
+{
+    FILE *f = fopen(filename, "wb");
+    if( f==NULL )
+        throw alglib::ap_error("file_put_contents: failed opening file");
+    if( fwrite((void*)contents, 1, strlen(contents), f)!=strlen(contents) )
+        throw alglib::ap_error("file_put_contents: failed writing to file");
+    fclose(f);
+}
+
 int main()
 {
     //
@@ -1075,6 +1085,139 @@ int main()
         if( !passed )
             return 1;
     }
+    
+    {
+        //
+        // Testing CSV functionality
+        //
+        const char *csv_name = "alglib-tst-35252-ndg4sf.csv";
+        bool passed = true;
+        try
+        {
+            // CSV_DEFAULT must be zero
+            passed = passed && alglib::CSV_DEFAULT==0;
+            
+            // absent file - must fail
+            try
+            {
+                alglib::real_2d_array arr;
+                read_csv("nonexistent123foralgtestinglib", '\t', alglib::CSV_DEFAULT, arr);
+                passed = false;
+            }
+            catch(alglib::ap_error)
+            { }
+            catch(...)
+            { passed = false; }
+            
+            // non-rectangular file - must fail
+            try
+            {
+                alglib::real_2d_array arr;
+                file_put_contents(csv_name, "a,b,c\r\n1,2");
+                read_csv(csv_name, ',', alglib::CSV_SKIP_HEADERS, arr);
+                remove(csv_name);
+                passed = false;
+            }
+            catch(alglib::ap_error)
+            { }
+            catch(...)
+            { passed = false; }
+            try
+            {
+                alglib::real_2d_array arr;
+                file_put_contents(csv_name, "a,b,c\r\n1,2,3,4");
+                read_csv(csv_name, ',', alglib::CSV_SKIP_HEADERS, arr);
+                remove(csv_name);
+                passed = false;
+            }
+            catch(alglib::ap_error)
+            { }
+            catch(...)
+            { passed = false; }
+            try
+            {
+                alglib::real_2d_array arr;
+                file_put_contents(csv_name, "1,2,3,4\n1,2,3\n1,2,3");
+                read_csv(csv_name, ',', alglib::CSV_DEFAULT, arr);
+                remove(csv_name);
+                passed = false;
+            }
+            catch(alglib::ap_error)
+            { }
+            catch(...)
+            { passed = false; }
+            
+            // empty file
+            try
+            {
+                alglib::real_2d_array arr;
+                file_put_contents(csv_name, "");
+                read_csv(csv_name, '\t', alglib::CSV_DEFAULT, arr);
+                remove(csv_name);
+                passed = passed && arr.rows()==0 && arr.cols()==0;
+            }
+            catch(...)
+            { passed = false; }
+            
+            // one row with header, tab separator
+            try
+            {
+                alglib::real_2d_array arr;
+                file_put_contents(csv_name, "a\tb\tc\n");
+                read_csv(csv_name, '\t', alglib::CSV_SKIP_HEADERS, arr);
+                remove(csv_name);
+                passed = passed && arr.rows()==0 && arr.cols()==0;
+            }
+            catch(...)
+            { passed = false; }
+            
+            // no header, comma-separated, full stop as decimal point
+            try
+            {
+                alglib::real_2d_array arr;
+                file_put_contents(csv_name, "1.5,2,3.25\n4,5,6");
+                read_csv(csv_name, ',', alglib::CSV_DEFAULT, arr);
+                remove(csv_name);
+                passed = passed && arr.tostring(2)=="[[1.50,2.00,3.25],[4.00,5.00,6.00]]";
+            }
+            catch(...)
+            { passed = false; }
+            
+            // header, tab-separated, mixed use of comma and full stop as decimal points
+            try
+            {
+                alglib::real_2d_array arr;
+                file_put_contents(csv_name, "a\tb\tc\n1.5\t2\t3,25\n4\t5.25\t6,1\n");
+                read_csv(csv_name, '\t', alglib::CSV_SKIP_HEADERS, arr);
+                remove(csv_name);
+                passed = passed && arr.tostring(2)=="[[1.50,2.00,3.25],[4.00,5.25,6.10]]";
+            }
+            catch(...)
+            { passed = false; }
+            
+            // header, tab-separated, fixed/exponential, spaces, mixed use of comma and full stop as decimal points
+            try
+            {
+                alglib::real_2d_array arr;
+                file_put_contents(csv_name, " a\t b \tc\n1,1\t 2.9\t -3.5  \n  1.1E1  \t 2.0E-1 \t-3E+1 \n+1  \t -2\t 3.    \n.1\t-.2\t+.3\n");
+                read_csv(csv_name, '\t', alglib::CSV_SKIP_HEADERS, arr);
+                remove(csv_name);
+                passed = passed && arr.tostring(2)=="[[1.10,2.90,-3.50],[11.00,0.20,-30.00],[1.00,-2.00,3.00],[0.10,-0.20,0.30]]";
+            }
+            catch(...)
+            { passed = false; }
+        }
+        catch(...)
+        { passed = false; }
+        
+        //
+        // Report
+        //
+        printf(fmt_str, "* CSV support", passed ? "OK" : "FAILED");
+        fflush(stdout);
+        if( !passed )
+            return 1;
+    }
         
     //
     // Testing issues which must be fixed
